/*
 *     create a mathematical puzzle
 *     output as xfig 3.2 format to stdout,
 *     use fig2dev from transfig to further conversion and printing
 *
 *     every symbol has to be replaced by a decimal digit in a unique way
 */

#include <stdlib.h>
#include <stdio.h>

char *version_string="CALCU v0.1 (c) Joerg Schulenburg, Sept. 2002";

int out_xfig(int rid[3][3]) {
   int i,j,k,bit,idx[10],x,x1,y1,x2,y2;
   for (i=0;i<10;i++) idx[i]=i;
   /* randomize idx */
   for (i=0;i<100;i++) { j=random()%10; x=idx[0]; idx[0]=idx[j]; idx[j]=x; }
   /* output xfig-head */
   printf("#FIG 3.2\nLandscape\nCenter\nMetric\nA4\n100.00\nSingle\n"
          "-2\n1200 2\n# created by %s\n",version_string);
   printf("# Summation Line, 3 thick\n"
          "2 1 0 3 0 7 50 0 -1 0.000 0 0 -1 0 0 2\n"
          "\t %5d %5d  %5d %5d\n",2000,3500,13000,3500);
   /* output symbols */
   for (i=0;i<3;i++) for (j=0;j<3;j++) {
     x=abs(rid[i][j]);
     /* print +-= */
     if (j>1)
       printf("# centered text, size 60, helvetica=16\n"
              "4 1 0 50 0 16 60 0.0000 4 375 375 %5d %5d =\\001\n",
              (j*4+1)*1000+500,i*2*1000+600);
     if (j==1)
       printf("# centered text, size 60, helvetica=16\n"
              "4 1 0 50 0 16 60 0.0000 4 375 375 %5d %5d %c\\001\n",
              (j*4+1)*1000+500,i*2*1000+600,((rid[i][j]<0)?'-':'+'));
     if (i==1)
       printf("# centered text, size 60, helvetica=16\n"
              "4 1 0 50 0 16 60 0.0000 4 375 375 %5d %5d %c\\001\n",
              (j*4+1)*1000+2500,i*2*1000-400,((rid[i][j]<0)?'-':'+'));
     for (k=0;x>0;k++,x/=10) {
       printf("# empty square\n"
              "2 2 0 1 0 7 50 0 -1 0.000 0 0 -1 0 0 5\n"
              "\t %5d %5d  %5d %5d  %5d %5d  %5d %5d  %5d %5d\n",
              (j*4+4-k)*1000    ,i*2*1000    ,(j*4+4-k)*1000+900,i*2*1000,
              (j*4+4-k)*1000+900,i*2*1000+900,(j*4+4-k)*1000    ,i*2*1000+900,
              (j*4+4-k)*1000    ,i*2*1000    );
       /* create pattern */
       for (bit=0; bit<4; bit++) {
         if(bit==0){ if(idx[x%10]&1){ x1=0; y1=0; x2=1; y2=0; } else continue; }
         if(bit==1){ if(idx[x%10]&2){ x1=1; y1=0; x2=1; y2=1; } else continue; }
         if(bit==2){ if(idx[x%10]&4){ x1=1; y1=1; x2=0; y2=1; } else continue; }
         if(bit==3){ if(idx[x%10]&8){ x1=0; y1=1; x2=0; y2=0; } else continue; }
         printf("# full black triangle\n"
              "2 3 0 1 0 0 40 0 20 0.000 0 0 -1 0 0 4\n"
              "\t %5d %5d  %5d %5d  %5d %5d  %5d %5d\n",
               (j*4+4-k)*1000+x1*900,2*i*1000+y1*900,
               (j*4+4-k)*1000+x2*900,2*i*1000+y2*900,
               (j*4+4-k)*1000   +450,2*i*1000+   450,
               (j*4+4-k)*1000+x1*900,2*i*1000+y1*900);
       }
       printf("# circ +450 +450 rad=350\n"
              "1 3 0 1 0 7 30 0 20 0.000 1 0.0000 %5d %5d %d %d %5d %5d %5d %5d\n",
              (j*4+4-k)*1000+450, i*2*1000+450, 350, 350,
              (j*4+4-k)*1000+450, i*2*1000+450,
              (j*4+4-k)*1000+800, i*2*1000+450);
     }
   }
}

#define BRUTE_FORCE 0
/* check all possible 2-diggit-riddles, brute force => 1:100 success */
int create_riddle(int rid[3][3]){
  int i=0,j=0,num=0,x,idx[10],i1,i2;
#if BRUTE_FORCE
  for (rid[2][2]=10; rid[2][2]<100; rid[2][2]++) {
    for (rid[2][0]=10; rid[2][0]<100; rid[2][0]++) {
      for (rid[0][2]=rid[2][0]; rid[0][2]<100; rid[0][2]++) {
#else
  for (;num==0;) { { {
        rid[2][2]=random()%990+10;
        rid[2][0]=random()%990+10;
        rid[0][2]=random()%990+10;
#endif
        rid[2][1]=rid[2][2]-rid[2][0];
        rid[1][2]=rid[2][2]-rid[0][2];
#if BRUTE_FORCE
        if (abs(rid[2][1])<10 || abs(rid[1][2])<10) continue; /* skip */
        for (rid[0][0]=10; rid[0][0]<100; rid[0][0]++){
#else
        for (i++;num==0 && (i&255)!=255;) {
          rid[0][0]=random()%990+10;
#endif
          rid[0][1]=rid[0][2]-rid[0][0];
          rid[1][0]=rid[2][0]-rid[0][0];
          rid[1][1]=abs(rid[2][1])-abs(rid[0][1]);
          x        =abs(rid[1][2])-abs(rid[1][0]);
          i++;
          if (x!=rid[1][1]) continue;
          j++;
          /* test if every digit 0..9 is used */
          for (i1=0;i1<10;i1++) idx[i1]=0; /* reset flags */
          for (i1=0;i1<3;i1++) for (i2=0;i2<3;i2++) {
            x=abs(rid[i1][i2]);
            idx[x%10]++;
            if (x> 9) idx[(x/ 10)%10]++;
            if (x>99) idx[(x/100)%10]++;
          }
          for (i2=i1=0;i1<10;i1++) if(!idx[i1])i2++; /* check flags */
          if (i2) continue;
          /* found a valid riddle */
          num++;
        }
        fprintf(stderr," tests: %8d %8d %8d\r",num,j,i);
      }
    }
  }
  fprintf(stderr,"\n");
  return num;
}

int main(int argn, char *argv[]){
  int i,j,rid[3][3];
  for (i=1;i<argn;i++)
    if (argv[i][0]=='-')
      if (argv[i][1]=='r') srand(atoi(argv[i]+2));
  create_riddle(rid);
  for (i=0;i<3;i++) {
    for (j=0;j<3;j++) {
      fprintf(stderr,"% 5d",rid[i][j]);
    }
    fprintf(stderr,"\n");
  }
  out_xfig(rid);
  return 0;
}
